@@ -1,49 +1,47 @@
-#!/usr/bin/env python3
 """
 Python interface to PyPI Stats API
 https://pypistats.org/api
 """
+from __future__ import annotations
+
 import atexit
 import datetime as dt
 import json
+import os
 import sys
 import warnings
 from pathlib import Path
 
-import httpx
-import pkg_resources
 from platformdirs import user_cache_dir
-from pytablewriter import (
-    HtmlTableWriter,
-    MarkdownTableWriter,
-    NumpyTableWriter,
-    PandasDataFrameWriter,
-    RstSimpleTableWriter,
-    String,
-    TsvTableWriter,
-)
-from pytablewriter.style import Align, Style, ThousandSeparator
 from slugify import slugify
+from termcolor import colored
 
-__version__ = pkg_resources.get_distribution(__name__).version
+try:
+    # Python 3.8+
+    import importlib.metadata as importlib_metadata
+except ImportError:
+    # Python 3.7 and lower
+    import importlib_metadata
+
+__version__ = importlib_metadata.version(__name__)
 
 BASE_URL = "https://pypistats.org/api/"
 CACHE_DIR = Path(user_cache_dir("pypistats"))
 USER_AGENT = f"pypistats/{__version__}"
 
 
-def _print_verbose(verbose, *args, **kwargs):
+def _print_verbose(verbose: bool, *args, **kwargs) -> None:
     """Print if verbose"""
     if verbose:
         _print_stderr(*args, **kwargs)
 
 
-def _print_stderr(*args, **kwargs):
+def _print_stderr(*args, **kwargs) -> None:
     """Print to stderr"""
     print(*args, file=sys.stderr, **kwargs)
 
 
-def _cache_filename(url):
+def _cache_filename(url: str) -> Path:
     """yyyy-mm-dd-url-slug.json"""
     today = dt.datetime.utcnow().strftime("%Y-%m-%d")
     slug = slugify(url)
@@ -52,7 +50,7 @@ def _cache_filename(url):
     return filename
 
 
-def _load_cache(cache_file):
+def _load_cache(cache_file: Path) -> dict:
     if not cache_file.exists():
         return {}
 
@@ -65,7 +63,7 @@ def _load_cache(cache_file):
     return data
 
 
-def _save_cache(cache_file, data):
+def _save_cache(cache_file: Path, data) -> None:
     try:
         if not CACHE_DIR.exists():
             CACHE_DIR.mkdir(parents=True)
@@ -77,7 +75,7 @@ def _save_cache(cache_file, data):
         pass
 
 
-def _clear_cache():
+def _clear_cache() -> None:
     """Delete old cache files, run as last task"""
     cache_files = CACHE_DIR.glob("**/*.json")
     this_month = dt.datetime.utcnow().strftime("%Y-%m")
@@ -90,14 +88,15 @@ atexit.register(_clear_cache)
 
 
 def pypi_stats_api(
-    endpoint,
-    params=None,
-    format="markdown",
-    start_date=None,
-    end_date=None,
-    sort=True,
-    total="all",
-    verbose=False,
+    endpoint: str,
+    params: str | None = None,
+    format: str = "markdown",
+    start_date: str | None = None,
+    end_date: str | None = None,
+    sort: bool = True,
+    total: str = "all",
+    color: str = "yes",
+    verbose: bool = False,
 ):
     """Call the API and return JSON"""
     if params:
@@ -116,6 +115,8 @@ def pypi_stats_api(
 
     if res == {}:
         # No cache, or couldn't load cache
+        import httpx
+
         r = httpx.get(url, headers={"User-Agent": USER_AGENT})
 
         # Raise if we made a bad request
@@ -171,6 +172,9 @@ def pypi_stats_api(
     data = _percent(data)
     data = _grand_total(data)
 
+    if color != "no" and format in ("markdown", "rst", "tsv") and _can_do_colour():
+        data = _colourify(data)
+
     output = _tabulate(data, format)
 
     if first and format not in ["numpy", "pandas"]:
@@ -198,7 +202,7 @@ def _filter(data, start_date=None, end_date=None):
     return data
 
 
-def _sort(data):
+def _sort(data: dict | list) -> dict | list:
     """Sort by downloads"""
 
     # Only for lists of dicts, not a single dict
@@ -209,7 +213,7 @@ def _sort(data):
     return data
 
 
-def _monthly_total(data):
+def _monthly_total(data: list) -> list:
     """Sum all downloads per category, by month"""
 
     totalled = {}
@@ -234,7 +238,7 @@ def _monthly_total(data):
     return data
 
 
-def _total(data):
+def _total(data: dict | list) -> dict | list:
     """Sum all downloads per category, regardless of date"""
 
     # Only for lists of dicts, not a single dict
@@ -255,7 +259,7 @@ def _total(data):
     return data
 
 
-def _date_range(data):
+def _date_range(data: dict | list) -> tuple[str | None, str | None]:
     """Return the first and last dates in data"""
     try:
         first = data[0]["date"]
@@ -273,7 +277,7 @@ def _date_range(data):
     return first, last
 
 
-def _grand_total_value(data):
+def _grand_total_value(data: list) -> int:
     """Return the grand total of the data"""
 
     # For "overall", without_mirrors is a subset of with_mirrors.
@@ -288,7 +292,7 @@ def _grand_total_value(data):
     return grand_total
 
 
-def _grand_total(data):
+def _grand_total(data: dict | list) -> dict | list:
     """Add a grand total row"""
 
     # Only for lists of dicts, not a single dict
@@ -307,7 +311,7 @@ def _grand_total(data):
     return data
 
 
-def _percent(data):
+def _percent(data: dict | list) -> dict | list:
     """Add a percent column"""
 
     # Only for lists of dicts, not a single dict
@@ -326,12 +330,96 @@ def _percent(data):
     return data
 
 
-def _tabulate(data, format="markdown"):
+def _can_do_colour() -> bool:
+    """Check https://no-color.org env vars and for dumb terminal"""
+    if "NO_COLOR" in os.environ:
+        return False
+    if "FORCE_COLOR" in os.environ:
+        return True
+    return (
+        hasattr(sys.stdout, "isatty")
+        and sys.stdout.isatty()
+        and os.environ.get("TERM") != "dumb"
+    )
+
+
+def _colourify(data) -> list:
+    """Add colour to percentages:
+    red: 0% - 5%
+    yellow: 6% - 15%
+    green: 15% - 100%
+    """
+
+    for row in data:
+        if "percent" not in row:
+            continue
+        percent = float(row["percent"].rstrip("%"))
+        if percent <= 5:
+            colour = "red"
+        elif percent <= 15:
+            colour = "yellow"
+        else:
+            colour = "green"
+        row["percent"] = colored(row["percent"], colour)
+    return data
+
+
+def _tabulate(data: dict | list, format: str = "markdown"):
     """Return data in specified format"""
 
+    if isinstance(data, dict):
+        headers = list(data.keys())
+    else:  # isinstance(data, list):
+        headers = sorted(set().union(*(d.keys() for d in data)))
+
+    # Move downloads last
+    headers.append("downloads")
+    headers.remove("downloads")
+
+    if format == "markdown":
+        return _prettytable(headers, data)
+    else:
+        return _pytablewriter(headers, data, format)
+
+
+def _prettytable(headers, data) -> str:
+    from prettytable import MARKDOWN, PrettyTable
+
+    x = PrettyTable()
+    x.set_style(MARKDOWN)
+
+    if isinstance(data, dict):
+        data = [data]
+    for header in headers:
+        col_data = [row[header] if header in row else "" for row in data]
+        x.add_column(header, col_data)
+        x.align["last_day"] = "r"
+        x.align["last_month"] = "r"
+        x.align["last_week"] = "r"
+        x.align["category"] = "l"
+        x.align["percent"] = "r"
+        x.align["downloads"] = "r"
+        x.custom_format["last_day"] = lambda f, v: f"{v:,}"
+        x.custom_format["last_month"] = lambda f, v: f"{v:,}"
+        x.custom_format["last_week"] = lambda f, v: f"{v:,}"
+        x.custom_format["downloads"] = lambda f, v: f"{v:,}"
+
+    return x.get_string() + "\n"
+
+
+def _pytablewriter(headers, data, format: str):
+    from pytablewriter import (
+        HtmlTableWriter,
+        NumpyTableWriter,
+        PandasDataFrameWriter,
+        RstSimpleTableWriter,
+        String,
+        TsvTableWriter,
+    )
+    from pytablewriter.style import Align, Style, ThousandSeparator
+
     format_writers = {
         "html": HtmlTableWriter,
-        "markdown": MarkdownTableWriter,
         "numpy": NumpyTableWriter,
         "pandas": PandasDataFrameWriter,
         "rst": RstSimpleTableWriter,
@@ -343,15 +431,10 @@ def _tabulate(data, format="markdown"):
         writer.margin = 1
 
     if isinstance(data, dict):
-        headers = list(data.keys())
         writer.value_matrix = [data]
     else:  # isinstance(data, list):
-        headers = sorted(set().union(*(d.keys() for d in data)))
         writer.value_matrix = data
 
-    # Move downloads last
-    headers.append("downloads")
-    headers.remove("downloads")
     writer.headers = headers
 
     # Custom alignment and format
@@ -386,7 +469,7 @@ def _tabulate(data, format="markdown"):
     return writer.dumps()
 
 
-def _paramify(param_name, param_value):
+def _paramify(param_name: str, param_value: float | str | None) -> str:
     """If param_value, return &param_name=param_value"""
     if isinstance(param_value, bool):
         param_value = str(param_value).lower()
@@ -397,14 +480,14 @@ def _paramify(param_name, param_value):
     return ""
 
 
-def recent(package, period=None, **kwargs):
+def recent(package: str, period: str | None = None, **kwargs: str):
     """Retrieve the aggregate download quantities for the last day/week/month"""
     endpoint = f"packages/{package}/recent"
     params = _paramify("period", period)
     return pypi_stats_api(endpoint, params, **kwargs)
 
 
-def overall(package, mirrors=None, **kwargs):
+def overall(package: str, mirrors: bool | str | None = None, **kwargs: str):
     """Retrieve the aggregate daily download time series with or without mirror
     downloads"""
     endpoint = f"packages/{package}/overall"
@@ -412,7 +495,7 @@ def overall(package, mirrors=None, **kwargs):
     return pypi_stats_api(endpoint, params, **kwargs)
 
 
-def python_major(package, version=None, **kwargs):
+def python_major(package: str, version: str | None = None, **kwargs: str):
     """Retrieve the aggregate daily download time series by Python major version
     number"""
     endpoint = f"packages/{package}/python_major"
@@ -420,7 +503,7 @@ def python_major(package, version=None, **kwargs):
     return pypi_stats_api(endpoint, params, **kwargs)
 
 
-def python_minor(package, version=None, **kwargs):
+def python_minor(package: str, version: str | None = None, **kwargs) -> str:
     """Retrieve the aggregate daily download time series by Python minor version
     number"""
     endpoint = f"packages/{package}/python_minor"
@@ -428,7 +511,7 @@ def python_minor(package, version=None, **kwargs):
     return pypi_stats_api(endpoint, params, **kwargs)
 
 
-def system(package, os=None, **kwargs):
+def system(package: str, os: str | None = None, **kwargs):
     """Retrieve the aggregate daily download time series by operating system"""
     endpoint = f"packages/{package}/system"
     params = _paramify("os", os)
