@@ -3,7 +3,9 @@ Unit tests for pypistats
 """
 import copy
 import json
+import os
 from pathlib import Path
+from unittest import mock
 
 import pytest
 import respx
@@ -62,28 +64,28 @@ SAMPLE_RESPONSE_OVERALL = """{
         }"""
 
 
-def stub__cache_filename(*args):
+def stub__cache_filename(*args) -> Path:
     return Path("/this/does/not/exist")
 
 
-def stub__save_cache(*args):
+def stub__save_cache(*args) -> None:
     pass
 
 
 class TestPypiStats:
-    def setup_method(self):
+    def setup_method(self) -> None:
         # Stub caching. Caches are tested in another class.
         self.original__cache_filename = pypistats._cache_filename
         self.original__save_cache = pypistats._save_cache
         pypistats._cache_filename = stub__cache_filename
         pypistats._save_cache = stub__save_cache
 
-    def teardown_method(self):
+    def teardown_method(self) -> None:
         # Unstub caching
         pypistats._cache_filename = self.original__cache_filename
         pypistats._save_cache = self.original__save_cache
 
-    def test__filter_no_filters_no_change(self):
+    def test__filter_no_filters_no_change(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
 
@@ -93,7 +95,7 @@ class TestPypiStats:
         # Assert
         assert data == output
 
-    def test__filter_start_date(self):
+    def test__filter_start_date(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
         start_date = "2018-09-22"
@@ -106,7 +108,7 @@ class TestPypiStats:
         for row in output:
             assert row["date"] >= start_date
 
-    def test__filter_end_date(self):
+    def test__filter_end_date(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
         end_date = "2018-04-22"
@@ -119,7 +121,7 @@ class TestPypiStats:
         for row in output:
             assert row["date"] <= end_date
 
-    def test__filter_start_and_end_date(self):
+    def test__filter_start_and_end_date(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
         start_date = "2018-09-01"
@@ -135,7 +137,7 @@ class TestPypiStats:
             assert row["date"] <= end_date
 
     @respx.mock
-    def test_warn_if_start_date_before_earliest_available(self):
+    def test_warn_if_start_date_before_earliest_available(self) -> None:
         # Arrange
         start_date = "2000-01-01"
         package = "pip"
@@ -161,7 +163,7 @@ class TestPypiStats:
             pypistats.python_major(package, start_date=start_date)
 
     @respx.mock
-    def test_error_if_end_date_before_earliest_available(self):
+    def test_error_if_end_date_before_earliest_available(self) -> None:
         # Arrange
         end_date = "2000-01-01"
         package = "pip"
@@ -186,57 +188,59 @@ class TestPypiStats:
         ):
             pypistats.python_major(package, end_date=end_date)
 
-    def test__paramify_none(self):
-        # Arrange
-        period = None
-
+    @pytest.mark.parametrize(
+        "test_name, test_value, expected",
+        [
+            ("period", None, ""),
+            ("period", "day", "&period=day"),
+            ("mirrors", True, "&mirrors=true"),
+            ("version", 3, "&version=3"),
+            ("version", 3.7, "&version=3.7"),
+        ],
+    )
+    def test__paramify(self, test_name, test_value, expected) -> None:
         # Act
-        param = pypistats._paramify("period", period)
+        param = pypistats._paramify(test_name, test_value)
 
         # Assert
-        assert param == ""
+        assert param == expected
 
-    def test__paramify_string(self):
+    @mock.patch.dict(os.environ, {"NO_COLOR": "TRUE"})
+    def test__can_do_colour_no_color(self) -> None:
+        # Act / Assert
+        assert pypistats._can_do_colour() is False
+
+    @mock.patch.dict(os.environ, {"FORCE_COLOR": "TRUE"})
+    def test__can_do_colour_force_colour(self) -> None:
+        # Act / Assert
+        assert pypistats._can_do_colour() is True
+
+    def test__colourify(self) -> None:
         # Arrange
-        period = "day"
+        data = [
+            {"category": "2.7", "downloads": 1},
+            {"category": "3.5", "downloads": 10},
+            {"category": "3.10", "downloads": 89},
+        ]
+        expected_output = [
+            # red
+            {"category": "2.7", "downloads": 1, "percent": "\x1b[31m1.00%\x1b[0m"},
+            # yellow
+            {"category": "3.5", "downloads": 10, "percent": "\x1b[33m10.00%\x1b[0m"},
+            # green
+            {"category": "3.10", "downloads": 89, "percent": "\x1b[32m89.00%\x1b[0m"},
+            {"category": "Total", "downloads": 100},
+        ]
+        data = pypistats._percent(data)
+        data = pypistats._grand_total(data)
 
         # Act
-        param = pypistats._paramify("period", period)
+        output = pypistats._colourify(data)
 
         # Assert
-        assert param == "&period=day"
+        assert output == expected_output
 
-    def test__paramify_bool(self):
-        # Arrange
-        mirrors = True
-
-        # Act
-        param = pypistats._paramify("mirrors", mirrors)
-
-        # Assert
-        assert param == "&mirrors=true"
-
-    def test__paramify_int(self):
-        # Arrange
-        version = 3
-
-        # Act
-        param = pypistats._paramify("version", version)
-
-        # Assert
-        assert param == "&version=3"
-
-    def test__paramify_float(self):
-        # Arrange
-        version = 3.7
-
-        # Act
-        param = pypistats._paramify("version", version)
-
-        # Assert
-        assert param == "&version=3.7"
-
-    def test__tabulate_noarg(self):
+    def test__tabulate_noarg(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA)
         expected_output = EXPECTED_TABULATED_MD
@@ -250,13 +254,13 @@ class TestPypiStats:
     @pytest.mark.parametrize(
         "test_input, expected",
         [
-            ("html", EXPECTED_TABULATED_HTML),
-            ("markdown", EXPECTED_TABULATED_MD),
-            ("rst", EXPECTED_TABULATED_RST),
-            ("tsv", EXPECTED_TABULATED_TSV),
+            pytest.param("html", EXPECTED_TABULATED_HTML, id="html"),
+            pytest.param("markdown", EXPECTED_TABULATED_MD, id="markdown"),
+            pytest.param("rst", EXPECTED_TABULATED_RST, id="rst"),
+            pytest.param("tsv", EXPECTED_TABULATED_TSV, id="tsv"),
         ],
     )
-    def test__tabulate(self, test_input, expected):
+    def test__tabulate(self, test_input: str, expected: str) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA)
 
@@ -266,7 +270,7 @@ class TestPypiStats:
         # Assert
         assert output.strip() == expected.strip()
 
-    def test__sort(self):
+    def test__sort(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA)
         expected_output = [
@@ -288,7 +292,7 @@ class TestPypiStats:
         # Assert
         assert output == expected_output
 
-    def test__sort_recent(self):
+    def test__sort_recent(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_RECENT)
 
@@ -298,7 +302,7 @@ class TestPypiStats:
         # Assert
         assert output == SAMPLE_DATA_RECENT
 
-    def test__monthly_total(self):
+    def test__monthly_total(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
 
@@ -316,7 +320,7 @@ class TestPypiStats:
         assert output[10]["downloads"] == 489_163
         assert output[10]["date"] == "2018-05"
 
-    def test__total(self):
+    def test__total(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
 
@@ -328,7 +332,7 @@ class TestPypiStats:
         assert output[0]["category"] == "2.4"
         assert output[0]["downloads"] == 9
 
-    def test__total_recent(self):
+    def test__total_recent(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_RECENT)
 
@@ -338,7 +342,7 @@ class TestPypiStats:
         # Assert
         assert output == SAMPLE_DATA_RECENT
 
-    def test__date_range(self):
+    def test__date_range(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
 
@@ -349,7 +353,7 @@ class TestPypiStats:
         assert first == "2018-04-16"
         assert last == "2018-09-23"
 
-    def test__date_range_no_dates_in_data(self):
+    def test__date_range_no_dates_in_data(self) -> None:
         # Arrange
         # recent
         data = [
@@ -367,7 +371,7 @@ class TestPypiStats:
         assert first is None
         assert last is None
 
-    def test__grand_total(self):
+    def test__grand_total(self) -> None:
         # Arrange
         data = copy.deepcopy(PYTHON_MINOR_DATA)
         original_len = len(data)
@@ -380,7 +384,7 @@ class TestPypiStats:
         assert output[-1]["category"] == "Total"
         assert output[-1]["downloads"] == 9_355_317
 
-    def test__grand_total_one_row(self):
+    def test__grand_total_one_row(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_ONE_ROW)
 
@@ -390,7 +394,7 @@ class TestPypiStats:
         # Assert
         assert output == SAMPLE_DATA_ONE_ROW
 
-    def test__grand_total_recent(self):
+    def test__grand_total_recent(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_RECENT)
 
@@ -400,7 +404,7 @@ class TestPypiStats:
         # Assert
         assert output == SAMPLE_DATA_RECENT
 
-    def test__percent(self):
+    def test__percent(self) -> None:
         # Arrange
         data = [
             {"category": "2.7", "downloads": 63749},
@@ -421,7 +425,7 @@ class TestPypiStats:
         # Assert
         assert output == expected_output
 
-    def test__percent_one_row(self):
+    def test__percent_one_row(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_ONE_ROW)
 
@@ -431,7 +435,7 @@ class TestPypiStats:
         # Assert
         assert output == SAMPLE_DATA_ONE_ROW
 
-    def test__percent_recent(self):
+    def test__percent_recent(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_RECENT)
 
@@ -442,7 +446,7 @@ class TestPypiStats:
         assert output == SAMPLE_DATA_RECENT
 
     @respx.mock
-    def test_valid_json(self):
+    def test_valid_json(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/recent?&period=day"
@@ -460,7 +464,29 @@ class TestPypiStats:
         assert json.loads(output) == json.loads(mocked_response)
 
     @respx.mock
-    def test_recent_tabular(self):
+    @pytest.mark.parametrize(
+        "test_format, expected_output",
+        [
+            pytest.param(
+                "markdown",
+                """
+|  last_day | last_month |  last_week |
+|----------:|-----------:|-----------:|
+| 2,295,765 | 67,759,913 | 15,706,750 |
+""",
+                id="markdown",
+            ),
+            pytest.param(
+                "tsv",
+                """
+"last_day"\t"last_month"\t"last_week"
+2,295,765\t67,759,913\t15,706,750
+""",
+                id="tsv",
+            ),
+        ],
+    )
+    def test_recent_tabular(self, test_format, expected_output) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/recent"
@@ -469,28 +495,23 @@ class TestPypiStats:
                 {"last_day": 2295765, "last_month": 67759913, "last_week": 15706750},
             "package": "pip", "type": "recent_downloads"
         }"""
-        expected_output = """
-| last_day  | last_month | last_week  |
-| --------: | ---------: | ---------: |
-| 2,295,765 | 67,759,913 | 15,706,750 |
-"""
 
         # Act
         respx.get(mocked_url).respond(content=mocked_response)
-        output = pypistats.recent(package)
+        output = pypistats.recent(package, format=test_format)
 
         # Assert
         assert output.strip() == expected_output.strip()
 
     @respx.mock
-    def test_overall_tabular_start_date(self):
+    def test_overall_tabular_start_date(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/overall?&mirrors=false"
         mocked_response = SAMPLE_RESPONSE_OVERALL
         expected_output = """
-|    category     | percent | downloads |
-| --------------- | ------: | --------: |
+| category        | percent | downloads |
+|:----------------|--------:|----------:|
 | with_mirrors    | 100.00% | 1,487,218 |
 | without_mirrors |  99.24% | 1,475,979 |
 | Total           |         | 1,487,218 |
@@ -506,14 +527,14 @@ Date range: 2020-05-02 - 2020-05-02
         assert output.strip() == expected_output.strip()
 
     @respx.mock
-    def test_overall_tabular_end_date(self):
+    def test_overall_tabular_end_date(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/overall?&mirrors=false"
         mocked_response = SAMPLE_RESPONSE_OVERALL
         expected_output = """
-|    category     | percent | downloads |
-| --------------- | ------: | --------: |
+| category        | percent | downloads |
+|:----------------|--------:|----------:|
 | with_mirrors    | 100.00% | 2,100,139 |
 | without_mirrors |  99.21% | 2,083,472 |
 | Total           |         | 2,100,139 |
@@ -529,7 +550,7 @@ Date range: 2020-05-01 - 2020-05-01
         assert output.strip() == expected_output.strip()
 
     @respx.mock
-    def test_python_major_json(self):
+    def test_python_major_json(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/python_major"
@@ -560,7 +581,7 @@ Date range: 2020-05-01 - 2020-05-01
         assert json.loads(output) == json.loads(expected_output)
 
     @respx.mock
-    def test_python_minor_json(self):
+    def test_python_minor_json(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/python_minor"
@@ -605,7 +626,7 @@ Date range: 2020-05-01 - 2020-05-01
         assert json.loads(output) == json.loads(expected_output)
 
     @respx.mock
-    def test_system_tabular(self):
+    def test_system_tabular(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/system"
@@ -621,8 +642,8 @@ Date range: 2020-05-01 - 2020-05-01
             "type": "system_downloads"
         }"""
         expected_output = """
-| category | percent |  downloads  |
-| -------- | ------: | ----------: |
+| category | percent |   downloads |
+|:---------|--------:|------------:|
 | Linux    |  83.14% | 236,502,274 |
 | null     |  10.75% |  30,579,325 |
 | Darwin   |   3.77% |  10,734,594 |
@@ -639,7 +660,7 @@ Date range: 2020-05-01 - 2020-05-01
         assert output.strip() == expected_output.strip()
 
     @respx.mock
-    def test_python_minor_monthly(self):
+    def test_python_minor_monthly(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/python_minor"
@@ -676,12 +697,12 @@ Date range: 2020-05-01 - 2020-05-01
         assert json.loads(output) == json.loads(expected_output)
 
     @respx.mock
-    def test_versions_are_strings(self):
+    def test_versions_are_strings(self) -> None:
         # Arrange
         data = copy.deepcopy(SAMPLE_DATA_VERSION_STRINGS)
         expected_output = """
 | category |    date    | downloads |
-| -------- | ---------- | --------: |
+|:---------|:----------:|----------:|
 | 3.1      | 2018-08-15 |        10 |
 | 3.10     | 2018-08-15 |         1 |
 """
@@ -694,7 +715,7 @@ Date range: 2020-05-01 - 2020-05-01
 
     @pytest.mark.skipif(numpy is None, reason="NumPy is not installed")
     @respx.mock
-    def test_format_numpy(self):
+    def test_format_numpy(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/overall"
@@ -715,7 +736,7 @@ Date range: 2020-05-01 - 2020-05-01
 
     @pytest.mark.skipif(pandas is None, reason="pandas is not installed")
     @respx.mock
-    def test_format_pandas(self):
+    def test_format_pandas(self) -> None:
         # Arrange
         package = "pip"
         mocked_url = "https://pypistats.org/api/packages/pip/overall"
